@@ -7,20 +7,23 @@ from amulet.api.block import Block
 from amulet.api.data_types import Dimension, OperationReturnType
 
 if TYPE_CHECKING:
-    from amulet.api.world import World
+    from amulet.api.level import BaseLevel
 
 
 def fill(
-    world: "World", dimension: Dimension, target_box: SelectionGroup, fill_block: Block
+    world: "BaseLevel",
+    dimension: Dimension,
+    target_box: SelectionGroup,
+    fill_block: Block,
 ) -> OperationReturnType:
     if not isinstance(fill_block, Block):
         raise Exception("Fill operation was not given a Block object")
-    internal_id = world.palette.get_add_block(fill_block)
+    internal_id = world.block_palette.get_add_block(fill_block)
 
-    iter_count = len(list(world.get_chunk_slices(target_box, dimension, True)))
+    iter_count = len(list(world.get_chunk_slice_box(dimension, target_box, True)))
     count = 0
 
-    for chunk, slices, _ in world.get_chunk_slices(target_box, dimension, True):
+    for chunk, slices, _ in world.get_chunk_slice_box(dimension, target_box, True):
         chunk.blocks[slices] = internal_id
         chunk.changed = True
         count += 1
